import { __decorate, __values, __spread, __metadata } from 'tslib';
import { NgForm, Validators } from '@angular/forms';
import { Injectable, ElementRef, Input, Directive, NgModule } from '@angular/core';
import { debounceTime, filter, tap } from 'rxjs/operators';
import { Subject } from 'rxjs';
import tippy from 'tippy.js';

var ValidatorConfig = /** @class */ (function () {
    function ValidatorConfig(identifier, validator, message, tooltipsOnInit, alwaysEvaluate) {
        if (message === void 0) { message = ""; }
        if (tooltipsOnInit === void 0) { tooltipsOnInit = false; }
        if (alwaysEvaluate === void 0) { alwaysEvaluate = false; }
        this.message = "";
        this.tooltipsOnInit = false;
        this.alwaysEvaluate = false;
        this.validator = validator;
        this.identifier = identifier;
        this.message = message,
            this.tooltipsOnInit = tooltipsOnInit;
        this.alwaysEvaluate = alwaysEvaluate;
    }
    return ValidatorConfig;
}());

var ValidatorId;
(function (ValidatorId) {
    ValidatorId["Required"] = "required";
    ValidatorId["Pattern"] = "pattern";
    ValidatorId["RequiredTrue"] = "required";
    ValidatorId["NullValidator"] = "nullvalidator";
    ValidatorId["MinLength"] = "minlength";
    ValidatorId["MaxLength"] = "maxlength";
    ValidatorId["Email"] = "email";
    ValidatorId["Max"] = "max";
    ValidatorId["Min"] = "min";
    ValidatorId["Between"] = "between";
    ValidatorId["InvalidValues"] = "invalidValues";
    ValidatorId["RequiredValues"] = "requiredValues";
    ValidatorId["InvalidCharacters"] = "invalidCharacters";
    ValidatorId["SqlObjectName"] = "sqlObjectName";
    ValidatorId["IsInteger"] = "isInteger";
    ValidatorId["IsNumeric"] = "isNumeric";
    ValidatorId["isAlphaNumeric"] = "isAlphaNumeric";
    ValidatorId["isAlphabetical"] = "isAlphabetical";
    ValidatorId["startsWith"] = "startsWith";
    ValidatorId["endsWith"] = "endsWith";
})(ValidatorId || (ValidatorId = {}));

var NgEasyValidationService = /** @class */ (function () {
    function NgEasyValidationService() {
        this.validationResultSubject = new Subject();
    }
    NgEasyValidationService.prototype.validate = function (requirements, formGroup) {
        var result = [];
        for (var field in formGroup.controls) {
            var control = formGroup.controls[field];
            if (!control.errors)
                continue;
            var _loop_1 = function (errorIdentifier) {
                var config = JSON.parse(JSON.stringify(requirements[field].find(function (x) { return x.identifier == errorIdentifier; })));
                this_1.setErrorMessage(field, control, config);
                result.push(new ValidationResult(field, config, !control.dirty && !config.tooltipsOnInit));
            };
            var this_1 = this;
            for (var errorIdentifier in control.errors) {
                _loop_1(errorIdentifier);
            }
        }
        ;
        this.validationResultSubject.next(result);
        return result;
    };
    NgEasyValidationService.prototype.setErrorMessage = function (field, control, config) {
        if (!config.message)
            config.message = this.generateErrorMessage(field, config.identifier, control.errors[config.identifier]);
        config.message = config.message.replace("<value>", control.value);
    };
    NgEasyValidationService.prototype.generateErrorMessage = function (field, requirementType, requirementDetails) {
        if (requirementDetails === void 0) { requirementDetails = null; }
        var errorMessage = "";
        field = field[0].toUpperCase() + field.substr(1);
        switch (requirementType) {
            case ValidatorId.Required:
                errorMessage = field + " is required";
                break;
            case ValidatorId.Pattern:
                errorMessage = field + " isn't in the right format. The format must be: " + requirementDetails.requiredPattern;
                break;
            case ValidatorId.Between:
                errorMessage = field + " must be between " + requirementDetails.min + " and " + requirementDetails.max;
                break;
            case ValidatorId.Email:
                errorMessage = field + " must be a valid e-mail address";
                break;
            case ValidatorId.Max:
                errorMessage = field + " must be lower than or equal to " + requirementDetails.max;
                break;
            case ValidatorId.Min:
                errorMessage = field + " must be higher than or equal to " + requirementDetails.min;
                break;
            case ValidatorId.RequiredTrue:
                errorMessage = field + " must be selected";
                break;
            case ValidatorId.MinLength:
                errorMessage = field + " has a minimum length of " + requirementDetails.requiredLength;
                break;
            case ValidatorId.MaxLength:
                errorMessage = field + " has a maximum length of " + requirementDetails.requiredLength;
                break;
            case ValidatorId.RequiredValues:
                var msg = this.humanizeList(requirementDetails.requiredValues);
                errorMessage = field + " must be " + msg;
                break;
            case ValidatorId.InvalidValues:
                var msg2 = this.humanizeList(requirementDetails.invalidValues);
                errorMessage = field + " can't be " + msg2;
                break;
            case ValidatorId.InvalidCharacters:
                var msg3 = this.humanizeList(requirementDetails.invalidCharacters);
                errorMessage = field + " can't contain " + msg3;
                break;
            case ValidatorId.SqlObjectName:
                errorMessage = field + " isn't a valid SQL name";
                break;
            case ValidatorId.IsInteger:
                errorMessage = field + " must be an integer";
                break;
            case ValidatorId.IsNumeric:
                errorMessage = field + " must be a numeric";
                break;
            case ValidatorId.isAlphaNumeric:
                errorMessage = field + " must be alpha numerical";
                break;
            case ValidatorId.isAlphabetical:
                errorMessage = field + " can only contain letters";
                break;
            case ValidatorId.startsWith:
                errorMessage = field + " must start with '" + requirementDetails.startsWith + "'";
                break;
            case ValidatorId.endsWith:
                errorMessage = field + " must end with '" + requirementDetails.endsWith + "'";
                break;
            default:
                errorMessage = field + " is invalid";
                break;
        }
        return errorMessage;
    };
    NgEasyValidationService.prototype.humanizeList = function (values) {
        var msg = "";
        for (var i = 0; i < values.length; i++) {
            if (i == values.length - 1)
                msg += " or ";
            msg += "'" + values[i] + "'";
            if (i < values.length - 2)
                msg += ", ";
        }
        return msg;
    };
    NgEasyValidationService = __decorate([
        Injectable()
    ], NgEasyValidationService);
    return NgEasyValidationService;
}());
var ValidationResult = /** @class */ (function () {
    function ValidationResult(field, validatorConfig, isHidden) {
        this.field = field;
        this.validatorConfig = validatorConfig;
        this.isHidden = isHidden;
    }
    return ValidationResult;
}());

var NgEasyValidationDirective = /** @class */ (function () {
    function NgEasyValidationDirective(el, validationService, host) {
        this.validationService = validationService;
        this.host = host;
        this.requirements = {};
        this.tippyProps = {};
        this.validationDebounceTime = 0;
        this.invalidClass = "validation-error";
        this.validationEnabled = false;
        this.tippies = {};
        this.formId = Math.random().toString(36).substring(2);
        el.nativeElement.setAttribute('formId', this.formId);
    }
    Object.defineProperty(NgEasyValidationDirective.prototype, "ngEasyValidation", {
        set: function (requirements) {
            var e_1, _a;
            var newRequirements = {};
            for (var field in requirements) {
                var configs = [];
                try {
                    //Convert passed configurations to ValidatorConfig if they are not already
                    for (var _b = (e_1 = void 0, __values(requirements[field])), _c = _b.next(); !_c.done; _c = _b.next()) {
                        var config = _c.value;
                        configs.push(new ValidatorConfig(config.validator.id, config.validator.validator, config.message, config.tooltipsOnInit || false, config.validator.alwaysEvaluate || false));
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
                newRequirements[field] = configs;
            }
            this.requirements = newRequirements;
        },
        enumerable: true,
        configurable: true
    });
    NgEasyValidationDirective.prototype.ngOnInit = function () {
        this.initValidationSubscription();
        this.setDefaultTippyProps();
    };
    NgEasyValidationDirective.prototype.initValidationSubscription = function () {
        var _this = this;
        this.validationSubscription = this.host.form.valueChanges
            .pipe(debounceTime(this.validationDebounceTime), filter(function (_) { return _this.validationEnabled; }), tap(this.validate.bind(this))).subscribe();
    };
    NgEasyValidationDirective.prototype.setDefaultTippyProps = function () {
        if (!this.tippyProps.allowHTML)
            this.tippyProps.allowHTML = true;
        tippy.setDefaultProps(this.tippyProps);
    };
    NgEasyValidationDirective.prototype.ngOnDestroy = function () {
        if (this.validationSubscription)
            this.validationSubscription.unsubscribe();
    };
    NgEasyValidationDirective.prototype.ngOnChanges = function (changes) {
        var _this = this;
        if (Object.keys(this.requirements).length == 0)
            this.validationEnabled = true;
        if (changes.ngEasyValidation) {
            setTimeout(function () {
                _this.ngEasyValidation = changes.ngEasyValidation.currentValue;
                _this.initFieldConfigurations();
                _this.host.form.updateValueAndValidity();
            }, 200);
        }
    };
    NgEasyValidationDirective.prototype.initFieldConfigurations = function () {
        var e_2, _a;
        if (!this.requirements)
            return;
        this.validationEnabled = false;
        var inputFields = this.getUsableInputFields(this.getFormElement());
        try {
            for (var inputFields_1 = __values(inputFields), inputFields_1_1 = inputFields_1.next(); !inputFields_1_1.done; inputFields_1_1 = inputFields_1.next()) {
                var element = inputFields_1_1.value;
                var field = element.getAttribute("name");
                var control = this.host.form.controls[field];
                var reqs = this.requirements[field];
                var isDirty = this.setFieldIsDirty(reqs, control);
                this.setFieldValidators(reqs, control);
                this.setFieldTippy(reqs, field, element, isDirty);
                control.updateValueAndValidity();
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (inputFields_1_1 && !inputFields_1_1.done && (_a = inputFields_1.return)) _a.call(inputFields_1);
            }
            finally { if (e_2) throw e_2.error; }
        }
        this.validationEnabled = true;
    };
    NgEasyValidationDirective.prototype.setFieldIsDirty = function (requirements, control) {
        var hasValue = control.value && control.value.length > 0;
        var mustValidateOnInit = requirements && requirements.some(function (x) { return x.tooltipsOnInit; });
        if (hasValue || mustValidateOnInit)
            control.markAsDirty();
        return hasValue || mustValidateOnInit;
    };
    NgEasyValidationDirective.prototype.setFieldValidators = function (requirements, control) {
        var reqsToAdd = [];
        reqsToAdd.push.apply(reqsToAdd, __spread(requirements.map(function (x) { return x.validator; })));
        control.clearValidators();
        control.setValidators(reqsToAdd);
    };
    NgEasyValidationDirective.prototype.setFieldTippy = function (requirements, field, element, isDirty) {
        var instance = this.tippies[field];
        if (!instance) {
            instance = tippy(element);
            this.tippies[field] = instance;
            if (!isDirty && !requirements.some(function (x) { return x.tooltipsOnInit; }))
                setTimeout(instance.disable, 10);
        }
    };
    NgEasyValidationDirective.prototype.validate = function () {
        var _this = this;
        var formEl = this.getFormElement();
        if (!formEl)
            return;
        var inputFields = this.getUsableInputFields(formEl);
        this.validationEnabled = false;
        this.reEvaluateFields(inputFields);
        setTimeout(function () { return _this.validationEnabled = true; }, this.validationDebounceTime);
        var validationResult = this.validationService.validate(this.requirements, this.host.form);
        this.updateFieldsLayout(inputFields, validationResult);
    };
    NgEasyValidationDirective.prototype.reEvaluateFields = function (inputFields) {
        var e_3, _a;
        var evaluateForm = false;
        try {
            for (var inputFields_2 = __values(inputFields), inputFields_2_1 = inputFields_2.next(); !inputFields_2_1.done; inputFields_2_1 = inputFields_2.next()) {
                var element = inputFields_2_1.value;
                var field = element.getAttribute("name");
                var control = this.host.form.controls[field];
                if (this.requirements[field].some(function (x) { return x.alwaysEvaluate; })) {
                    control.updateValueAndValidity();
                    evaluateForm = true;
                }
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (inputFields_2_1 && !inputFields_2_1.done && (_a = inputFields_2.return)) _a.call(inputFields_2);
            }
            finally { if (e_3) throw e_3.error; }
        }
        if (evaluateForm)
            this.host.form.updateValueAndValidity();
    };
    NgEasyValidationDirective.prototype.updateFieldsLayout = function (inputFields, allErrors) {
        var e_4, _a;
        var _loop_1 = function (el) {
            var field = el.getAttribute("name");
            var errors = allErrors.filter(function (x) { return x.field == field; });
            var hasError = errors.length == 0 || errors.some(function (x) { return x.isHidden; });
            this_1.updateFieldTippy(el, hasError, field, errors);
            this_1.updateFieldLayout(el, hasError);
        };
        var this_1 = this;
        try {
            for (var inputFields_3 = __values(inputFields), inputFields_3_1 = inputFields_3.next(); !inputFields_3_1.done; inputFields_3_1 = inputFields_3.next()) {
                var el = inputFields_3_1.value;
                _loop_1(el);
            }
        }
        catch (e_4_1) { e_4 = { error: e_4_1 }; }
        finally {
            try {
                if (inputFields_3_1 && !inputFields_3_1.done && (_a = inputFields_3.return)) _a.call(inputFields_3);
            }
            finally { if (e_4) throw e_4.error; }
        }
    };
    NgEasyValidationDirective.prototype.updateFieldTippy = function (el, hasError, field, errors) {
        var tippyInstance = this.tippies[field];
        if (hasError) {
            tippyInstance.disable();
        }
        else {
            var errorMessage = this.buildErrorMessage(errors);
            tippyInstance.setContent(errorMessage);
            tippyInstance.enable();
            if (this.isMouseOnElement(el))
                tippyInstance.show();
        }
    };
    NgEasyValidationDirective.prototype.updateFieldLayout = function (el, hasError) {
        var hasClass = el.classList.contains(this.invalidClass);
        if (hasError && !hasClass) {
            el.classList.add(this.invalidClass);
        }
        else if (!hasError && hasClass) {
            el.classList.remove(this.invalidClass);
        }
    };
    NgEasyValidationDirective.prototype.isMouseOnElement = function (element) {
        var hoverEls = document.querySelectorAll("input:hover, textarea:hover, select:hover");
        var hoverEl = hoverEls[hoverEls.length - 1];
        return hoverEl == element && element == document.activeElement;
    };
    NgEasyValidationDirective.prototype.buildErrorMessage = function (validationResults) {
        var e_5, _a;
        var errorMessage = "";
        var errorMessages = validationResults.map(function (x) { return x.validatorConfig.message || "This field is incorrect"; }) || [];
        if (errorMessages.length > 1) {
            errorMessage += "<ul style='list-style-position:outside;padding-left:18px;margin:0'>";
            try {
                for (var errorMessages_1 = __values(errorMessages), errorMessages_1_1 = errorMessages_1.next(); !errorMessages_1_1.done; errorMessages_1_1 = errorMessages_1.next()) {
                    var msg = errorMessages_1_1.value;
                    errorMessage += "<li>" + msg + "</li>";
                }
            }
            catch (e_5_1) { e_5 = { error: e_5_1 }; }
            finally {
                try {
                    if (errorMessages_1_1 && !errorMessages_1_1.done && (_a = errorMessages_1.return)) _a.call(errorMessages_1);
                }
                finally { if (e_5) throw e_5.error; }
            }
            errorMessage += "</ul>";
        }
        if (errorMessages.length == 1) {
            errorMessage = errorMessages[0];
        }
        return errorMessage;
    };
    NgEasyValidationDirective.prototype.getFormElement = function () {
        return document.querySelectorAll('[formId="' + this.formId + '"]')[0];
    };
    NgEasyValidationDirective.prototype.getUsableInputFields = function (formElement) {
        var _this = this;
        var inputFields = formElement.querySelectorAll("input, textarea, select");
        var filteredInputFields = Array.from(inputFields).filter(function (x) { return Object.keys(_this.requirements).includes(x.getAttribute("name")); });
        return filteredInputFields;
    };
    NgEasyValidationDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: NgEasyValidationService },
        { type: NgForm }
    ]; };
    __decorate([
        Input("ngEasyValidation"),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], NgEasyValidationDirective.prototype, "ngEasyValidation", null);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], NgEasyValidationDirective.prototype, "tippyProps", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Number)
    ], NgEasyValidationDirective.prototype, "validationDebounceTime", void 0);
    NgEasyValidationDirective = __decorate([
        Directive({
            selector: '[ngEasyValidation]',
            providers: [NgEasyValidationService]
        }),
        __metadata("design:paramtypes", [ElementRef,
            NgEasyValidationService,
            NgForm])
    ], NgEasyValidationDirective);
    return NgEasyValidationDirective;
}());

var ValidatorInfo = /** @class */ (function () {
    function ValidatorInfo(validator, id, alwaysEvaluate) {
        if (alwaysEvaluate === void 0) { alwaysEvaluate = false; }
        this.validator = validator;
        this.id = id;
        this.alwaysEvaluate = alwaysEvaluate;
    }
    return ValidatorInfo;
}());

//Wrapper methods around Angular's validators
function required() {
    return new ValidatorInfo(Validators.required, ValidatorId.Required);
}
function pattern(regex) {
    return new ValidatorInfo(Validators.pattern(regex), ValidatorId.Pattern);
}
function requiredTrue() {
    return new ValidatorInfo(Validators.requiredTrue, ValidatorId.Required);
}
function nullValidator() {
    return new ValidatorInfo(Validators.required, ValidatorId.NullValidator);
}
function minLength(minLength) {
    return new ValidatorInfo(Validators.minLength(minLength), ValidatorId.MinLength);
}
function maxLength(maxLength) {
    return new ValidatorInfo(Validators.maxLength(maxLength), ValidatorId.MaxLength);
}
function email() {
    return new ValidatorInfo(Validators.required, ValidatorId.Email);
}
function min(min) {
    return new ValidatorInfo(Validators.min(min), ValidatorId.Min);
}
function max(max) {
    return new ValidatorInfo(Validators.max(max), ValidatorId.Max);
}
//Custom validators
function between(min, max) {
    var res = function (control) {
        return control.value && (control.value > max || control.value < min) ? { 'between': { value: control.value, max: max, min: min } } : null;
    };
    return new ValidatorInfo(res, ValidatorId.Between);
}
function invalidValues(invalidValues, valueModifier) {
    if (valueModifier === void 0) { valueModifier = function (x) { return x; }; }
    var res = function (control) {
        var controlValue = valueModifier(control.value);
        if (!(invalidValues instanceof Array))
            invalidValues = [invalidValues];
        return invalidValues.map(function (x) { return typeof x == "string" ? x.toLowerCase() : x; }).includes(typeof controlValue == "string" ? controlValue.toLowerCase() : controlValue) ? { 'invalidValues': { value: controlValue, invalidValues: invalidValues } } : null;
    };
    return new ValidatorInfo(res, ValidatorId.InvalidValues);
}
function requiredValues(requiredValues, valueModifier) {
    if (valueModifier === void 0) { valueModifier = function (x) { return x; }; }
    var res = function (control) {
        var controlValue = valueModifier(control.value);
        if (!(requiredValues instanceof Array))
            requiredValues = [requiredValues];
        return !requiredValues.map(function (x) { return typeof x == "string" ? x.toLowerCase() : x; }).includes(typeof controlValue == "string" ? controlValue.toLowerCase() : controlValue) ? { 'requiredValues': { value: controlValue, requiredValues: requiredValues } } : null;
    };
    return new ValidatorInfo(res, ValidatorId.RequiredValues);
}
function invalidCharacters(invalidCharacters) {
    var regex = "^[^";
    for (var i = 0; i < invalidCharacters.length; i++) {
        var c = invalidCharacters[i];
        if ("[\^$.|?*+()".includes(c))
            regex += "\\";
        regex += c;
    }
    regex += "]*$";
    var res = function (control) {
        return !(new RegExp(regex, "i")).test(control.value) ? { 'invalidCharacters': { value: control.value, invalidCharacters: invalidCharacters } } : null;
    };
    return new ValidatorInfo(res, ValidatorId.InvalidCharacters);
}
function sqlObjectName() {
    var res = function (control) {
        return (control.value && !(new RegExp("^[_a-zA-Zß#][0-9a-zA-Zß_@$#]*$", "i")).test(control.value)) ? { 'sqlObjectName': { value: control.value } } : null;
    };
    return new ValidatorInfo(res, ValidatorId.SqlObjectName);
}
function conditionalValidator(validator, condition) {
    var res = function (control) {
        if (!condition(control))
            return null;
        return validator.validator(control);
    };
    return new ValidatorInfo(res, validator.id, true);
}
function isInteger() {
    var res = function (control) {
        var value = control.value || "";
        return !(new RegExp("^[0-9]*$", "i")).test(value) ? { 'isInteger': { value: value } } : null;
    };
    return new ValidatorInfo(res, ValidatorId.IsInteger);
}
function isNumeric() {
    var res = function (control) {
        var value = control.value || "";
        return !(new RegExp("^[0-9]*[.,]?[0-9]*$", "i")).test(value) ? { 'isNumeric': { value: value } } : null;
    };
    return new ValidatorInfo(res, ValidatorId.IsNumeric);
}
function isAlphaNumeric() {
    var res = function (control) {
        var value = control.value || "";
        var code, i, len;
        for (i = 0, len = value.length; i < len; i++) {
            code = value.charCodeAt(i);
            if (!(code > 47 && code < 58) && // numeric (0-9)
                !(code > 64 && code < 91) && // upper alpha (A-Z)
                !(code > 96 && code < 123)) { // lower alpha (a-z)
                return { 'isAlphaNumeric': { value: value } };
            }
        }
        return null;
    };
    return new ValidatorInfo(res, ValidatorId.isAlphaNumeric);
}
function isAlphabetical() {
    var res = function (control) {
        var value = control.value || "";
        var code, i, len;
        for (i = 0, len = value.length; i < len; i++) {
            code = value.charCodeAt(i);
            if (!(code > 64 && code < 91) && // upper alpha (A-Z)
                !(code > 96 && code < 123)) { // lower alpha (a-z)
                return { 'isAlphabetical': { value: value } };
            }
        }
        return null;
    };
    return new ValidatorInfo(res, ValidatorId.isAlphabetical);
}
function startsWith(str) {
    var res = function (control) {
        var value = control.value || "";
        return !value || value.startsWith(str) ? null : { "startsWith": { value: value, startsWith: str } };
    };
    return new ValidatorInfo(res, ValidatorId.startsWith);
}
function endsWith(str) {
    var res = function (control) {
        var value = control.value || "";
        return !value || value.endsWith(str) ? null : { "endsWith": { value: value, endsWith: str } };
    };
    return new ValidatorInfo(res, ValidatorId.endsWith);
}

var NgEasyValidationModule = /** @class */ (function () {
    function NgEasyValidationModule() {
    }
    NgEasyValidationModule = __decorate([
        NgModule({
            imports: [],
            declarations: [
                NgEasyValidationDirective
            ],
            providers: [
                NgEasyValidationService
            ],
            exports: [
                NgEasyValidationDirective
            ]
        })
    ], NgEasyValidationModule);
    return NgEasyValidationModule;
}());

/*
 * Public API Surface of ng-easy-validation
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgEasyValidationDirective, NgEasyValidationModule, ValidatorInfo, between, conditionalValidator, email, endsWith, invalidCharacters, invalidValues, isAlphaNumeric, isAlphabetical, isInteger, isNumeric, max, maxLength, min, minLength, nullValidator, pattern, required, requiredTrue, requiredValues, sqlObjectName, startsWith, NgEasyValidationService as ɵa, ValidatorId as ɵb };
//# sourceMappingURL=ng-easy-validation.js.map
